@@ -1,5 +1,5 @@
 /*  Clzip - LZMA lossless data compressor
-    Copyright (C) 2010-2016 Antonio Diaz Diaz.
+    Copyright (C) 2010-2017 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -49,6 +49,7 @@ enum {
   min_dictionary_size = 1 << min_dictionary_bits,	/* >= modeled_distances */
   max_dictionary_bits = 29,
   max_dictionary_size = 1 << max_dictionary_bits,
+  min_member_size = 36,
   literal_context_bits = 3,
   literal_pos_state_bits = 0,				/* not used */
   pos_state_bits = 2,
@@ -131,9 +132,9 @@ static inline void Pp_init( struct Pretty_print * const pp,
   pp->stdin_name = "(stdin)";
   pp->longest_name = 0;
   pp->first_post = false;
-  stdin_name_len = strlen( pp->stdin_name );
 
   if( verbosity <= 0 ) return;
+  stdin_name_len = strlen( pp->stdin_name );
   for( i = 0; i < num_filenames; ++i )
     {
     const char * const s = filenames[i];
@@ -182,8 +183,10 @@ static inline void CRC32_update_buf( uint32_t * const crc,
                                      const int size )
   {
   int i;
+  uint32_t c = *crc;
   for( i = 0; i < size; ++i )
-    *crc = crc32[(*crc^buffer[i])&0xFF] ^ ( *crc >> 8 );
+    c = crc32[(c^buffer[i])&0xFF] ^ ( c >> 8 );
+  *crc = c;
   }
 
 
@@ -243,7 +246,7 @@ static inline bool Fh_set_dictionary_size( File_header data, const unsigned sz )
     {
     const unsigned base_size = 1 << data[5];
     const unsigned fraction = base_size / 16;
-    int i;
+    unsigned i;
     for( i = 7; i >= 1; --i )
       if( base_size - ( i * fraction ) >= sz )
         { data[5] |= ( i << 5 ); break; }
@@ -290,14 +293,30 @@ static inline void Ft_set_member_size( File_trailer data, unsigned long long sz
   { int i; for( i = 12; i <= 19; ++i ) { data[i] = (uint8_t)sz; sz >>= 8; } }
 
 
+static const char * const bad_magic_msg = "Bad magic number (file not in lzip format).";
+static const char * const bad_dict_msg = "Invalid dictionary size in member header.";
+static const char * const trailing_msg = "Trailing data not allowed.";
+
 /* defined in decoder.c */
 int readblock( const int fd, uint8_t * const buf, const int size );
 int writeblock( const int fd, const uint8_t * const buf, const int size );
 
+/* defined in list.c */
+int list_files( const char * const filenames[], const int num_filenames,
+                const bool ignore_trailing );
+
 /* defined in main.c */
 extern int verbosity;
+struct stat;
+const char * bad_version( const unsigned version );
+const char * format_ds( const unsigned dictionary_size );
+int open_instream( const char * const name, struct stat * const in_statsp,
+                   const bool no_ofile, const bool reg_only );
+void * resize_buffer( void * buf, const unsigned min_size );
 void cleanup_and_fail( const int retval );
 void show_error( const char * const msg, const int errcode, const bool help );
+void show_file_error( const char * const filename, const char * const msg,
+                      const int errcode );
 void internal_error( const char * const msg );
 struct Matchfinder_base;
 void show_progress( const unsigned long long partial_size,
